--[[
File: Config.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

local selectedRole
local selectedSet = {}
local labels = {}

config = Interface.Create("Context", "Config")
config:SetSecureMode("restricted")

local function OnEquipChange(equip)
	if equip.equipment:GetValue() then
		equip.itemsLabel:SetAlpha(1)
		equip.items:SetEnabled(true)
	else
		equip.itemsLabel:SetAlpha(0.5)
		equip.items:SetEnabled(false)
		if equip.items:GetText() ~= "" then
			equip.items:SetText("")
		end
	end
end

local function UpdateRole()
	local role = roles[selectedRole]
	role:SetMenu(config.menu:GetChecked())
	role:SetCustomIcon(config.customIcon:GetValue())
	
	local custom = {}
	for i, line in ipairs(config.custom:GetLines()) do
		line = string.ltrim(line, "/")
		if string.len(line) > 0 then
			table.insert(custom, line)
		end
	end
	role:SetCustom(custom)
	
	for _, button in ipairs({"Left", "Right"}) do
		role:SetEquipment(button, config.equip[button].equipment:GetValue())
		role:SetItems(button, config.equip[button].items:GetLines())
		role:SetWardrobe(button, config.equip[button].wardrobe:GetValue())
	end
	
	if KaruulAlert() then
		role:SetAlertSet(config.kAlertSet:GetValue())
		role:SetAlertSubset(config.kAlertSubset:GetValue())
	end
end

local function OnCustomIconSearchTextChange(currently_saved)
	local text = labels.customIconSearch:GetText()
	
	if currently_saved ~= nil then
		current_value = currently_saved
	else
		current_value = config.customIcon:GetValue()
	end
	
	local customIconItems = {
		{value = "none", label = L.config.roles.selectNone},
	}
	
	for index = 1, #customIcons.Labels do
		if (#customIconItems <= 60 and string.find(string.lower(customIcons.Labels[index].label), string.lower(text))) 
		or customIcons.Labels[index].value == current_value then
			customIconItems[#customIconItems + 1] = {
				value = customIcons.Labels[index].value, 
				label = customIcons.Labels[index].label,
				texture = customIcons.Textures[customIcons.Labels[index].value]
			}
		end
	end

	config.customIcon:SetItems(customIconItems)
	config.customIcon:SetValue(current_value)
end

local function OnRoleChange()
	selectedRole = config.role:GetValue()
	local role = roles[selectedRole]
	
	labels.customIconSearch:SetText("")
	OnCustomIconSearchTextChange(role:GetCustomIcon())
	config.menu:SetChecked(role:GetMenu())
	config.custom:SetLines(role:GetCustom())
	
	for _, button in ipairs({"Left", "Right"}) do
		local equip = config.equip[button]
		equip.equipment:SetValue(role:GetEquipment(button))
		OnEquipChange(equip)
		equip.items:SetLines(role:GetItems(button))
		equip.wardrobe:SetValue(role:GetWardrobe(button))
	end

	if KaruulAlert() then
		config.kAlertSet:SetValue(role:GetAlertSet())
		config.kAlertSubset:SetValue(role:GetAlertSubset())
	end
end

local InitializeTab = {}

local function CreateLabel(text, parent, heading)
	local label = Interface.Create("Text", "Label", parent)
	label:SetText(text)
	if heading then
		label:SetFontSize(14)
		label:SetFontColor(0.96, 0.87, 0.74, 1)
		label:SetEffectGlow({colorR = 0.54, colorG = 0.42, colorB = 0.22, strength = 1})
	else
		label:SetFontSize(13)
	end
	return label
end

local function GetSetLabel(index, name)
	local label = string.format(L.config.roles.selectSet, index)
	if name then
		label = string.format("%s - %s", label, name)
	end
	return label
end

InitializeTab[L.config.tabRoles] = function(tab)
	config.roleSection = Interface.Create("Canvas", "RoleSection", tab)
	config.roleSection:SetMargins(25, 25, 25, 25)
	config.roleSection:SetBackgroundImage("sub_window_02.png.dds")
	config.roleSection:SetPoint("TOPLEFT", tab, "TOPLEFT", 0, 8)
	config.roleSection:SetPoint("RIGHT", tab, "RIGHT")
	config.roleSection:SetHeight(66 + 20)
	config.roleSection:SetLayer(4)

	config.role = Interface.Create("RiftSelect", "Role", config.roleSection)
	config.role:SetPoint("TOPLEFT", config.roleSection, "TOPLEFT", 108, 9)
	config.role:SetWidth(224)
	config.role:SetLayer(3)
	config.role.OnChange = function() UpdateRole() OnRoleChange() end

	labels.role = CreateLabel(L.config.roles.labelRole, config.roleSection)
	labels.role:SetPoint(1, 0.5, config.role, 0, 0.5, -4, 1)
	
	config.menu = Interface.Create("RiftCheckbox", "Menu", config.roleSection)
	config.menu:SetPoint(0, 0.5, config.role, 1, 0.5, 16, 0)
	config.menu:SetLabel(L.config.roles.checkboxMenu)
	
	local customIconItems = {{value = "none", label = L.config.roles.selectNone}}
	config.customIcon = Interface.Create("RiftIconSelect", "CustomIcon", config.roleSection)
	config.customIcon:SetPoint("TOPLEFT", config.role, "BOTTOMLEFT", 0, 3)
	config.customIcon:SetWidth(224)
	config.customIcon:SetScrollbar(20)
	config.customIcon:SetItems(customIconItems)
	
	labels.customIcon = CreateLabel(L.config.roles.labelSetCustomIcon, config.roleSection)
	labels.customIcon:SetPoint(1, 0.5, config.customIcon, 0, 0.5, -4, 1)
	
	labels.customIconSearch = Interface.Create("RiftTextfield", "IconSearch", config.roleSection)
	labels.customIconSearch:SetPoint("TOPLEFT", config.customIcon, "TOPRIGHT", 10, 0)
	labels.customIconSearch:SetPoint("BOTTOMLEFT", config.customIcon, "BOTTOMRIGHT", 10, 0)
	labels.customIconSearch:SetWidth(90)
	labels.customIconSearch:SetMaximumLines(1)
	labels.customIconSearch.OnChange = OnCustomIconSearchTextChange
	
	config.equip = {}
	local equipmentItems = {{value = nil, label = L.config.roles.selectNone}}
	for index = 1, 20 do
		equipmentItems[index + 1] =
		{
			value = index,
			label = GetSetLabel(index, GetSetName("equipment", index))
		}
	end
	local wardrobeItems = {{value = nil, label = L.config.roles.selectNone}, {value = 0, label = L.config.roles.selectEquipment}}
	for index = 1, 39 do
		wardrobeItems[index + 2] =
		{
			value = index,
			label = GetSetLabel(index, GetSetName("wardrobe", index))
		}
	end
	
	for _, button in ipairs({"Left", "Right"}) do
		config.equip[button] = Interface.Create("Frame", "Equip" .. button, tab)
		local equip = config.equip[button]
		equip:SetLayer(3)
		
		equip:SetPoint(string.upper(button), tab, string.upper(button))
		equip:SetPoint("TOP", config.roleSection, "BOTTOM", nil, 10)
		equip:SetWidth(252)
		
		equip.clickLabel = CreateLabel(L.config.roles[string.format("section%sClick", button)], equip, true)
		equip.clickLabel:SetPoint("TOPLEFT", equip, "TOPLEFT")
		
		equip.equipmentLabel = CreateLabel(L.config.roles.labelLoadEquipment, equip)
		equip.equipmentLabel:SetPoint("TOPLEFT", equip.clickLabel, "BOTTOMLEFT", 0, 4)
		equip.equipment = Interface.Create("RiftSelect", "EquipSelect" .. button, equip)
		equip.equipment:SetLayer(2)
		equip.equipment:SetScrollbar(8)
		equip.equipment:SetItems(equipmentItems)
		equip.equipment:SetPoint("TOPLEFT", equip.equipmentLabel, "BOTTOMLEFT")
		equip.equipment:SetPoint("RIGHT", equip, "RIGHT")
		equip.equipment.OnChange = function() OnEquipChange(equip) end
		
		equip.itemsLabel = CreateLabel(L.config.roles.labelReplacementItems, equip)
		equip.itemsLabel:SetPoint("TOPLEFT", equip.equipment, "BOTTOMLEFT", 0, 4)
		equip.items = Interface.Create("RiftTextfield", "EquipItems", equip)
		equip.items:SetPoint("TOPLEFT", equip.itemsLabel, "BOTTOMLEFT", 0, 2)
		equip.items:SetPoint("RIGHT", equip, "RIGHT")
		equip.items:SetHeight(92)
		equip.items:SetMaximumLines(6)
		
		equip.wardrobeLabel = CreateLabel(L.config.roles.labelLoadWardrobe, equip)
		equip.wardrobeLabel:SetPoint("TOPLEFT", equip.items, "BOTTOMLEFT", 0, 4)
		equip.wardrobe = Interface.Create("RiftSelect", "WardrobeSelect" .. button, equip)
		equip.wardrobe:SetScrollbar(8)
		equip.wardrobe:SetItems(wardrobeItems)
		equip.wardrobe:SetPoint("TOPLEFT", equip.wardrobeLabel, "BOTTOMLEFT")
		equip.wardrobe:SetPoint("RIGHT", equip, "RIGHT")
		
		equip:SetPoint("BOTTOM", equip.wardrobe, "BOTTOM")
	end
	
	labels.custom = CreateLabel(L.config.roles.labelAdditionalMacroText, tab)
	labels.custom:SetPoint("TOPLEFT", config.equip.Left, "BOTTOMLEFT", 0, 10)
	config.custom = Interface.Create("RiftTextfield", "Custom", tab)
	config.custom:SetPoint("TOPLEFT", labels.custom, "BOTTOMLEFT", 0, 2)
	config.custom:SetHeight(92)
	config.custom:SetMaximumLines(6)
	
	if KaruulAlert() then
		config.custom:SetPoint("RIGHT", tab, "RIGHT", -186, nil)

		local kAlertSetItems = {{value = nil, label = L.config.roles.selectNone}, {value = "auto", label = L.config.roles.selectAuto}}
		for index = 1, #roles do
			kAlertSetItems[index + 2] =
			{
				value = index,
				label = string.format(L.config.roles.selectSet, index)
			}
		end
		labels.kAlertSet = CreateLabel(L.config.roles.labelLoadKaruulAlertSet, tab)
		labels.kAlertSet:SetPoint("TOP", labels.custom, "TOP")
		labels.kAlertSet:SetPoint("LEFT", config.custom, "RIGHT", 12, nil)
		config.kAlertSet = Interface.Create("RiftSelect", "kAlertSet", tab)
		config.kAlertSet:SetLayer(2)
		config.kAlertSet:SetScrollbar(6)
		config.kAlertSet:SetItems(kAlertSetItems)
		config.kAlertSet:SetPoint("TOPLEFT", labels.kAlertSet, "BOTTOMLEFT")
		config.kAlertSet:SetPoint("RIGHT", tab, "RIGHT")
		
		local kAlertSubsetItems = {{value = nil, label = L.config.roles.selectNone}, {value = 0, label = L.config.roles.selectDisable}}
		for index = 1, 10 do
			kAlertSubsetItems[index + 2] =
			{
				value = index,
				label = string.format(L.config.roles.selectSubset, index)
			}
		end
		labels.kAlertSubset = CreateLabel(L.config.roles.labelLoadKaruulAlertSubset, tab)
		labels.kAlertSubset:SetPoint("TOPLEFT", config.kAlertSet, "BOTTOMLEFT", 0, 9)
		config.kAlertSubset = Interface.Create("RiftSelect", "kAlertSubset", tab)
		config.kAlertSubset:SetScrollbar(6)
		config.kAlertSubset:SetItems(kAlertSubsetItems)
		config.kAlertSubset:SetPoint("TOPLEFT", labels.kAlertSubset, "BOTTOMLEFT")
		config.kAlertSubset:SetPoint("RIGHT", tab, "RIGHT")
	else
		config.custom:SetPoint("RIGHT", tab, "RIGHT")
	end	
end

local setsTab

local function UpdateSetName(type)
	local name = string.trim(config[type].name:GetText())
	if string.len(name) == 0 then
		name = nil
	end
	
	if GetSetName(type, selectedSet[type]) ~= name then
		SetSetName(type, selectedSet[type], name)

		local index = selectedSet[type]
		if type == "equipment" then
			index = index + 1
		elseif type == "wardrobe" then
			index = index + 2
		end
		
		for _, button in ipairs({"Left", "Right"}) do
			config.equip[button][type]:SetItemLabel(GetSetLabel(selectedSet[type], name), index)
		end
	end
end

local function SetButtonMacros(type)
	if not type or type == "equipment" then
		setsTab.overlay.equipmentLoadButton:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("loadequip %i", selectedSet["equipment"]))
		setsTab.overlay.equipmentSaveButton:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("saveequip %i", selectedSet["equipment"]))
	end
	if not type or type == "wardrobe" then
		setsTab.overlay.wardrobeLoadButton:EventMacroSet(Event.UI.Input.Mouse.Left.Click, string.format("wardrobe %i", selectedSet["wardrobe"]))
	end
end

local function OnSetChange(type)
	selectedSet[type] = config[type]:GetValue()
	local name = GetSetName(type, selectedSet[type])
	if name then
		config[type].name:SetText(name)
	else
		config[type].name:SetText("")
	end
	
	if setsTab.overlay and setsTab.overlay:GetVisible() then
		SetButtonMacros(type)
	end
end

local function OnSetNameChange(type)
	local name = string.trim(config[type].name:GetText())
	if string.len(name) == 0 then
		name = nil
	end
	config[type]:SetItemLabel(GetSetLabel(selectedSet[type], name))
end

local function OnMouseOverButton(button)
	button.macro:SetText(string.format("/%s", button:EventMacroGet(Event.UI.Input.Mouse.Left.Click)))
end

local function OnMouseOutButton(button)
	button.macro:SetText("")
end

InitializeTab[L.config.tabSets] = function(tab)
	setsTab = tab
	
	local function GetSetItems(type, count)
		local items = {}
		for index = 1, count do
			items[index] =
			{
				value = index,
				label = GetSetLabel(index, GetSetName(type, index))
			}
		end
		return items
	end

	labels.equipment = CreateLabel(L.config.sets.sectionEquipment, tab, true)
	labels.equipment:SetPoint("TOPLEFT", tab, "TOPLEFT", 0, 60)
	
	config.equipment = Interface.Create("RiftSelect", "Equipment", tab)
	config.equipment:SetLayer(2)
	config.equipment:SetPoint("TOPLEFT", labels.equipment, "BOTTOMLEFT", 0, 6)
	config.equipment:SetWidth(240)
	config.equipment:SetScrollbar(10)
	config.equipment:SetItems(GetSetItems("equipment", 20))
	config.equipment.OnChange = function() UpdateSetName("equipment") OnSetChange("equipment") end
	
	setsTab.equipmentLoadButton = Interface.Create("RiftButton", "EquipmentLoadBase", tab)
	setsTab.equipmentLoadButton:SetPoint("CENTERLEFT", config.equipment, "CENTERRIGHT", 8, 0)
	setsTab.equipmentLoadButton:SetText(L.config.sets.buttonLoad)
	setsTab.equipmentLoadButton:SetEnabled(false)

	setsTab.equipmentSaveButton = Interface.Create("RiftButton", "EquipmentSaveBase", tab)
	setsTab.equipmentSaveButton:SetPoint("CENTERLEFT", setsTab.equipmentLoadButton, "CENTERRIGHT")
	setsTab.equipmentSaveButton:SetText(L.config.sets.buttonSave)
	setsTab.equipmentSaveButton:SetEnabled(false)
	
	labels.equipmentSetName = CreateLabel(L.config.sets.labelNameOfSet, tab)
	labels.equipmentSetName:SetPoint("TOPLEFT", config.equipment, "BOTTOMLEFT", 0, 8)
	
	config.equipment.name = Interface.Create("RiftTextfield", "equipmentSetName", tab)
	config.equipment.name:SetPoint("TOPLEFT", labels.equipmentSetName, "BOTTOMLEFT", 0, 2)
	config.equipment.name:SetSize(240, 28)
	config.equipment.name:SetMaximumLines(1)
	config.equipment.name.OnChange = function() OnSetNameChange("equipment") end
	
	labels.wardrobe = CreateLabel(L.config.sets.sectionWardrobe, tab, true)
	labels.wardrobe:SetPoint("TOPLEFT", config.equipment.name, "BOTTOMLEFT", 0, 60)
	
	config.wardrobe = Interface.Create("RiftSelect", "Wardrobe", tab)
	config.wardrobe:SetPoint("TOPLEFT", labels.wardrobe, "BOTTOMLEFT", 0, 6)
	config.wardrobe:SetWidth(240)
	config.wardrobe:SetScrollbar(10)
	config.wardrobe:SetItems(GetSetItems("wardrobe", 39))
	config.wardrobe.OnChange = function() UpdateSetName("wardrobe") OnSetChange("wardrobe") end
	
	setsTab.wardrobeLoadButton = Interface.Create("RiftButton", "WardrobeLoadBase", tab)
	setsTab.wardrobeLoadButton:SetPoint("CENTERLEFT", config.wardrobe, "CENTERRIGHT", 8, 0)
	setsTab.wardrobeLoadButton:SetText(L.config.sets.buttonLoad)
	setsTab.wardrobeLoadButton:SetEnabled(false)
	
	labels.wardrobeSetName = CreateLabel(L.config.sets.labelNameOfSet, tab)
	labels.wardrobeSetName:SetPoint("TOPLEFT", config.wardrobe, "BOTTOMLEFT", 0, 8)
	
	config.wardrobe.name = Interface.Create("RiftTextfield", "wardrobeSetName", tab)
	config.wardrobe.name:SetPoint("TOPLEFT", labels.wardrobeSetName, "BOTTOMLEFT", 0, 2)
	config.wardrobe.name:SetSize(240, 28)
	config.wardrobe.name:SetMaximumLines(1)
	config.wardrobe.name.OnChange = function() OnSetNameChange("wardrobe") end

	config.equipment:SetValue(1)
	config.wardrobe:SetValue(1)
	OnSetChange("equipment")
	OnSetChange("wardrobe")

	tab.OnShow = function()
		if not Inspect.System.Secure() then
			if not setsTab.overlay then
				setsTab.overlay = Interface.Create("Frame", setsTab:GetName() .. ".Overlay", config)
				setsTab.overlay:SetLayer(config.dialog:GetLayer() + 1)
				setsTab.overlay:SetSecureMode("restricted")
	
				local function CreateSetButton(setType, buttonType)
					local buttonName = string.format("%s%sButton", setType, buttonType)
					setsTab.overlay[buttonName] = Interface.Create("RiftButton", buttonName, setsTab.overlay)
					setsTab.overlay[buttonName]:SetPoint("TOPLEFT", setsTab[buttonName], "TOPLEFT")
					setsTab.overlay[buttonName]:SetText(L.config.sets["button" .. buttonType])
					setsTab.overlay[buttonName]:SetSecureMode("restricted")
					setsTab.overlay[buttonName]:EventAttach(Event.UI.Input.Mouse.Cursor.In, OnMouseOverButton, "Event.UI.Input.Mouse.Cursor.In")
					setsTab.overlay[buttonName]:EventAttach(Event.UI.Input.Mouse.Cursor.Out, OnMouseOutButton, "Event.UI.Input.Mouse.Cursor.Out")
					setsTab.overlay[buttonName].macro = CreateLabel("", setsTab.overlay)
					setsTab.overlay[buttonName].macro:SetPoint("BOTTOMCENTER", setsTab.overlay[buttonName], "TOPCENTER")
				end
				
				CreateSetButton("equipment", "Load")
				CreateSetButton("equipment", "Save")
				CreateSetButton("wardrobe", "Load")
			end
			setsTab.overlay:SetVisible(true)
			SetButtonMacros()
		end
	end

	tab.OnHide = function()
		if setsTab.overlay and setsTab.overlay:GetVisible() then
			setsTab.overlay:SetVisible(false)
		end
		UpdateSetName("equipment")
		UpdateSetName("wardrobe")
	end

	Command.Event.Attach(Event.System.Secure.Leave, function()
		if config.dialog and config.dialog:GetVisible() and setsTab:GetVisible() then
			setsTab.OnShow()
		end
	end, "Event.System.Secure.Leave")

	Command.Event.Attach(Event.System.Secure.Enter, function()
		setsTab.OnHide()
	end, "Event.System.Secure.Enter")
end

function UpdateLockButton()
	if config.lock then
		if saved.config.lock then
			config.lock:SetText(L.config.settings.buttonUnlock)
		else
			config.lock:SetText(L.config.settings.buttonLock)
		end
	end
end

InitializeTab[L.config.tabSettings] = function(tab)
	local spacing = 28

	labels.sort = CreateLabel(L.config.settings.labelOrdering, tab)
	labels.sort:SetPoint("TOPRIGHT", tab, "TOPLEFT", 142, 60)
	
	config.sort = Interface.Create("RiftSelect", "Sort", tab)
	local items = {}
	for i, sort in ipairs({"Slot", "Name", "Type"}) do
		items[i] = {value = sort, label = L.config.settings["selectSort" .. sort]}
	end
	config.sort:SetItems(items)
	config.sort:SetWidth(280)
	config.sort:SetPoint("CENTERLEFT", labels.sort, "CENTERRIGHT", 12, -1)
	config.sort:SetLayer(2)
	config.sort:SetValue(saved.config.sort)
	config.sort.OnChange = function(self) saved.config.sort = self:GetValue() end
	
	labels.extend = CreateLabel(L.config.settings.labelDirection, tab)
	labels.extend:SetPoint("TOPRIGHT", labels.sort, "BOTTOMRIGHT", 0, spacing)
	
	config.extend = Interface.Create("RiftSelect", "Extend", tab)
	items = {}
	for i, extend in ipairs({"Down", "Up", "Right", "Left"}) do
		items[i] = {value = extend, label = L.config.settings["selectExtend" .. extend]}
	end
	config.extend:SetItems(items)
	config.extend:SetWidth(280)
	config.extend:SetPoint("CENTERLEFT", labels.extend, "CENTERRIGHT", 12, -1)
	config.extend:SetValue(saved.config.extend)
	config.extend.OnChange = function(self) saved.config.extend = self:GetValue() end
	
	labels.spacing = CreateLabel(L.config.settings.labelSpacing, tab)
	labels.spacing:SetPoint("TOPRIGHT", labels.extend, "BOTTOMRIGHT", 0, spacing)

	config.spacing = Interface.Create("RiftSlider", "Spacing", tab)
	config.spacing:SetPoint("CENTERLEFT", labels.spacing, "CENTERRIGHT", 14, 8)
	config.spacing:SetWidth(276)
	config.spacing:SetRange(constants.minSpacing, constants.maxSpacing)
	config.spacing:SetPosition(saved.config.spacing)
	config.spacing:EventAttach(Event.UI.Slider.Change, function(self)
		saved.config.spacing = self:GetPosition()
		menu:GetList():SetSpacing(saved.config.spacing)
	end, "SpacingSliderChange")
	
	labels.width = CreateLabel(L.config.settings.labelWidth, tab)
	labels.width:SetPoint("TOPRIGHT", labels.spacing, "BOTTOMRIGHT", 0, spacing)
	
	config.width = Interface.Create("RiftSlider", "Width", tab)
	config.width:SetPoint("CENTERLEFT", labels.width, "CENTERRIGHT", 14, 8)
	config.width:SetWidth(276)
	config.width:SetRange(constants.minWidth, constants.maxWidth)
	config.width:SetPosition(saved.config.width)
	config.width:EventAttach(Event.UI.Slider.Change, function(self)
		saved.config.width = self:GetPosition()
		menu:SetWidth(saved.config.width)
	end, "WidthSliderChange")
	
	labels.fade = CreateLabel(L.config.settings.labelInactiveAlpha, tab)
	labels.fade:SetPoint("TOPRIGHT", labels.width, "BOTTOMRIGHT", 0, spacing)
	
	config.fade = Interface.Create("RiftSlider", "InactiveAlpha", tab)
	config.fade:SetPoint("CENTERLEFT", labels.fade, "CENTERRIGHT", 14, 8)
	config.fade:SetWidth(276)
	config.fade:SetRange(constants.minFade, constants.maxFade)
	config.fade:SetPosition(saved.config.fade)
	config.fade:EventAttach(Event.UI.Slider.Change, function(self)
		saved.config.fade = self:GetPosition()
		menu:SetFade(saved.config.fade)
	end, "FadeSliderChange")
	
	labels.location = CreateLabel(L.config.settings.labelLocation, tab)
	labels.location:SetPoint("TOPRIGHT", labels.fade, "BOTTOMRIGHT", 0, spacing)
	
	config.lock = Interface.Create("RiftButton", "Lock", tab)
	config.lock:SetPoint("CENTERLEFT", labels.location, "CENTERRIGHT", 14, 0)
	UpdateLockButton()
	config.lock:EventAttach(Event.UI.Input.Mouse.Left.Click, function() SlashCommand(nil, "lock") end, "ClickLock")
	
	config.reset = Interface.Create("RiftButton", "Reset", tab)
	config.reset:SetPoint("CENTERLEFT", config.lock, "CENTERRIGHT", 6, 0)
	config.reset:SetText(L.config.settings.buttonReset)
	config.reset:EventAttach(Event.UI.Input.Mouse.Left.Click, function() SlashCommand(nil, "reset") end, "ClickReset")
	
	config.background = Interface.Create("RiftCheckbox", "Background", tab)
	config.background:SetLabel(L.config.settings.checkboxBackground)
	config.background:SetPoint("TOPLEFT", labels.location, "BOTTOMRIGHT", 12, spacing + 4)
	config.background:SetChecked(saved.config.background)
	config.background:EventAttach(Event.UI.Checkbox.Change, function(self)
		saved.config.background = self:GetChecked()
		menu:ShowBackground(saved.config.background)
	end, "BackgroundCheckboxChange")

	config.notification = Interface.Create("RiftCheckbox", "Notification", tab)
	config.notification:SetLabel(L.config.settings.checkboxNotification)
	config.notification:SetPoint("TOPLEFT", config.background, "BOTTOMLEFT", 0, 18)
	config.notification:SetChecked(saved.config.notification)
	config.notification:EventAttach(Event.UI.Checkbox.Change, function(self)
		saved.config.notification = self:GetChecked()
	end, "NotificationCheckboxChange")
end

local sections =
{
	{header = L.config.help.sectionRoleIcons, text = L.config.help.helpRoleIcons},
	{header = L.config.help.sectionRoleNames, text = L.config.help.helpRoleNames},
	{header = L.config.help.sectionEquipmentSets, text = L.config.help.helpEquipmentSets},
	{header = L.config.help.sectionRoleMenu, text = L.config.help.helpRoleMenu}
}

InitializeTab[L.config.tabHelp] = function(tab)
	for index, section in ipairs(sections) do
		section.headerLabel = CreateLabel(section.header, tab, true)
		if index == 1 then
			section.headerLabel:SetPoint("TOPLEFT", tab, "TOPLEFT", 0, 6)
		else
			section.headerLabel:SetPoint("TOPLEFT", sections[index - 1].textLabel, "BOTTOMLEFT", 0, 10)
		end
		section.textLabel = CreateLabel(section.text, tab)
		section.textLabel:SetPoint("TOPLEFT", section.headerLabel, "BOTTOMLEFT")
		section.textLabel:SetPoint("RIGHT", tab, "RIGHT")
		section.textLabel:SetWordwrap(true)
	end
	if addon:IsOutdated() then
		labels.version = CreateLabel(string.format(L.config.help.versionOutdated, addon.toc.Version), tab)
		labels.version:SetFontColor(1, 0, 0, 1)
	else
		labels.version = CreateLabel(string.format(L.config.help.version, addon.toc.Version), tab)
	end
	labels.version:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", 0, 0)
	labels.author = CreateLabel(string.format(L.config.help.author, addon.toc.Author), tab)
	labels.author:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", 0, 0)
end

local function OnInitializeTab(tab, name)
	InitializeTab[name](tab)
end

local function InitializeDialog(self)
	self:SetTitle(L.config.dialogTitle)
	self:SetSize(620, 660)

	self.tabview = Interface.Create("RiftTabview", "Tabview", self)
	self.tabview.OnInitializeTab = OnInitializeTab
	self.tabview:SetPoint("TOPLEFT", self, "TOPLEFT", 30, 70)
	self.tabview:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -30, -28)

	for _, tab in ipairs({L.config.tabRoles, L.config.tabSets, L.config.tabSettings, L.config.tabHelp}) do
		self.tabview:AddTab(tab)
	end

	function self:OnClose()
		if setsTab then
			setsTab.OnHide()
		end
		UpdateRole()
		if self:GetParent().OnClose then
			self:GetParent().OnClose()
		end
	end
end

function config:Show()
	if not self.dialog then
		self.dialog = Interface.Create("RiftWindow", "Dialog", self)
		self.dialog:SetVisible(false)
		InitializeDialog(self.dialog)
		self.dialog:Center()
	end

	local items = {}
	for index, role in ipairs(roles) do
		items[index] = {value = index, label = string.format("%i - %s", index, role:GetName())}
	end
	self.role:SetItems(items)
	self.role:SetValue(GetCurrentRole():GetSlot())
	OnRoleChange()

	self.dialog:SetVisible(true)
	if setsTab and setsTab:GetVisible() then
		setsTab.OnShow()
	end
end

function config:IsOpen()
	return self.dialog and self.dialog:GetVisible()
end
