--[[
File: Interface/List.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

Interface.List = setmetatable({}, Interface.Frame)
local List = Interface.List
List.__index = List

List.type = "Frame"

function List:Initialize()
	self.count = 0
	self.items = {}
	self.margin = 4
	self.spacing = 0
end

local function SetItemPoints(self, index)
	if index == 1 then
		self.items[index]:SetPoint("TOPLEFT", self, "TOPLEFT", self.margin, self.margin)
	end
	if self.vertical then
		if index > 1 then
			self.items[index]:SetPoint("TOPLEFT", self.items[index - 1], "BOTTOMLEFT")
		end
		self.items[index]:SetPoint("RIGHT", self, "RIGHT", -self.margin, nil)
	else
		if index > 1 then
			self.items[index]:SetPoint("TOPLEFT", self.items[index - 1], "TOPRIGHT")
		end
		self.items[index]:SetPoint("BOTTOM", self, "BOTTOM", nil, -self.margin)
	end
end

local function UpdateItemSize(self, index)
	if self.vertical then
		self.items[index]:SetHeight(self.spacing)
	else
		self.items[index]:SetWidth(self.spacing)
	end
end

local function UpdateSize(self)
	local size = self.spacing * self.count + self.margin * 2
	if self.vertical then
		self:SetHeight(size)
	else
		self:SetWidth(size)
	end
end

local function ResetTooltip(self)
	self.tooltip.ref = 0
	self.tooltip:SetVisible(false)
end

local function GetTooltip(self)
	if self.vertical then
		return nil
	else
		if not self.tooltip then
			self.tooltip = Interface.Create("Frame", "Tooltip", self)
			self.tooltip:SetBackgroundColor(self:GetBackgroundColor())
			self.tooltip:SetLayer(-1)
			ResetTooltip(self)
		end
		return self.tooltip
	end
end

function List:SetOrientation(vertical)
	if self.vertical ~= vertical then
		self.vertical = vertical
		if self.tooltip then
			ResetTooltip(self)
		end
		for index = 1, #self.items do
			self.items[index]:ClearAll()
			SetItemPoints(self, index)
			UpdateItemSize(self, index)
			if self.UpdateItem then
				self.UpdateItem(self.items[index], GetTooltip(self))
			end
		end
		UpdateSize(self)
	end
end

function List:SetSpacing(spacing)
	self.spacing = spacing
	for index = 1, #self.items do
		UpdateItemSize(self, index)
	end
	UpdateSize(self)
end

function List:SetCount(count)
	self.count = count
	for index, item in ipairs(self.items) do
		item:SetVisible(index <= self.count)
	end
	UpdateSize(self)
end

local function OnLeftClick(item)
	local self = item:GetParent()
	if self.OnItemClick then
		self.OnItemClick(item, "Left")
	end
end

local function OnRightClick(item)
	local self = item:GetParent()
	if self.OnItemClick then
		self.OnItemClick(item, "Right")
	end
end

local function OnHighlight(item, highlight)
	local self = item:GetParent()
	local tooltip = GetTooltip(self)

	if self.OnItemHighlight then
		self.OnItemHighlight(item, highlight, tooltip)
	end

	if tooltip then
		if highlight then
			tooltip:ClearPoint("TOPCENTER")
			tooltip:SetPoint("BOTTOMCENTER", item, "TOPCENTER", 0, -self.margin * 2)
			if tooltip:GetTop() < 0 then
				tooltip:ClearPoint("BOTTOMCENTER")
				tooltip:SetPoint("TOPCENTER", item, "BOTTOMCENTER", 0, self.margin * 2)
			end
			tooltip.ref = math.min(tooltip.ref + 1, 2)
		else
			tooltip.ref = math.max(tooltip.ref - 1, 0)
		end
		tooltip:SetVisible(tooltip.ref > 0)
	end
end

local function CreateItem(self, index)
	self.items[index] = Interface.Create("Frame", "Item" .. index, self)
	local item = self.items[index]
	SetItemPoints(self, index)
	UpdateItemSize(self, index)
	
	item.OnHighlight = OnHighlight
	item:EnableHighlight(true)
	item.OnLeftClick = OnLeftClick
	item:EnableClick(true, "Left")
	item.OnRightClick = OnRightClick
	item:EnableClick(true, "Right")
	
	item:SetSecureMode("restricted")
	if self.InitializeItem then
		self.InitializeItem(item)
	end
	if self.UpdateItem then
		self.UpdateItem(item, GetTooltip(self))
	end
end

function List:GetItem(index)
	if not self.items[index] then
		for i = #self.items + 1, index do
			CreateItem(self, i)
		end
	end
	return self.items[index]
end
