--[[
File: Addon.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))
Addon = {}
Addon.__index = Addon

local addons = {}
local message, time
local messageIdentifier = select(1, ...).identifier .. ".Version"
local broadcastTimer = Timer.Create(600)

local function IsVersion(version)
	version = version or ""
	return string.match(version, "^%d+%.%d+$") or string.match(version, "^%d+%.%d+%.%d+$")
end

local versionMetatable =
{
	__eq = function(v1, v2)
		return v1.major == v2.major and v1.minor == v2.minor and v1.patch == v2.patch
	end,
	__lt = function(v1, v2)
		if v1.major ~= v2.major then
			return v1.major < v2.major
		elseif v1.minor ~= v2.minor then
			return v1.minor < v2.minor
		else
			return v1.patch < v2.patch
		end
	end,
	__le = function(v1, v2)
		if v1.major ~= v2.major then
			return v1.major < v2.major
		elseif v1.minor ~= v2.minor then
			return v1.minor < v2.minor
		else
			return v1.patch <= v2.patch
		end
	end
}

local function Version(version)
	version = string.split(version, ".")
	return setmetatable({major = tonumber(version[1]), minor = tonumber(version[2]), patch = tonumber(version[3]) or 0}, versionMetatable)
end

function Addon.Register(addon)
	setmetatable(addon, Addon)
	addons[addon.identifier] = addon

	addon._version = {}
	local version = string.split(addon.toc.Version, "-")
	if IsVersion(version[1]) then
		addon._version.number = version[1]		
		if #version == 1 then
			addon._version.build = "release"
		elseif version[2] == "beta" or version[2] == "alpha" then
			addon._version.build = version[2]
		end
	else
		addon._version.build = "dev"
	end
	
	if addon._version.build == "release" then
		if not message then
			message = string.format("%s:%s", addon.identifier, addon._version.number)
		else
			message = string.format("%s|%s:%s", message, addon.identifier, addon._version.number)
		end
	end
end

broadcastTimer.OnElapsed = function()
	Command.Message.Broadcast("yell", nil, messageIdentifier, message)
end

Command.Event.Attach(Event.Addon.Startup.End, function()
	if message then
		broadcastTimer:Start()
	end
end, "Event.Addon.Startup.End")

Command.Message.Accept(nil, messageIdentifier)

Command.Event.Attach(Event.Message.Receive, function(_, _, _, _, identifier, data)
	if identifier == messageIdentifier then
		for _, addon in ipairs(string.split(data, "|")) do
			addon = string.split(addon, ":")
			if addons[addon[1]] and IsVersion(addon[2]) then
				local global = addons[addon[1]]:GetSavedVariables("global")
				if global and (not global.latestVersion or not IsVersion(global.latestVersion) or Version(global.latestVersion) < Version(addon[2])) then
					global.latestVersion = addon[2]
				end
			end
		end
	end
end, "Event.Message.Receive")

function Addon:IsOutdated()
	local latestVersion = self:GetSavedVariables("global").latestVersion
	if latestVersion and IsVersion(latestVersion) then
		if self._version.build == "release" then
			return Version(self._version.number) < Version(latestVersion)
		elseif self._version.build == "beta" or self._version.build == "alpha" then
			return Version(self._version.number) <= Version(latestVersion)
		end
	end
	return false
end

function Addon:GetSavedVariables(scope)
	if self.toc.SavedVariables then
		local variable = string.format("%s_%s", self.identifier, scope)
		if self.toc.SavedVariables[variable] == scope then
			_G[variable] = _G[variable] or {}
			return _G[variable]
		end
	end
	return nil
end

Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, function(_, identifier)
	if addons[identifier] then
		for variable, scope in pairs(addons[identifier].toc.SavedVariables) do
			if variable == string.format("%s_%s", identifier, scope) then
				_G[variable].version = addons[identifier].toc.Version
			end
		end
	end
end, "Addon.SavedVariables.Save.Begin")
