--[[
File: Roles.lua
Author: Zormus
--]]

setfenv(1, select(2, ...))

roles = {}
local player, currentRole

local function UpdateMenu(update)
	if menu:IsOpen() or update then
		table.sort(roles, Compare[saved.config.sort])
		local count = 0
		for _, role in ipairs(roles) do
			if role:GetMenu() then
				count = count + 1
				menu:UpdateItem(count, role)
			end
		end
		menu:SetCount(count)
		table.sort(roles, Compare.Slot)
	end
end

function menu:OnOpen()
	self:SetLayout(saved.config.extend)
	UpdateRoleList()
	self:UpdateButton(currentRole)
	UpdateMenu(true)
end

local function ShowConfig()
	if not config:IsOpen() then
		menu:SetActive(false)
		UpdateRoleList()
		menu:UpdateButton(currentRole)
		config:Show()
	end
end

function config:OnClose()
	for _, role in ipairs(roles) do
		role:UpdateMacros()
	end
	if not Inspect.System.Secure() then
		menu:SetActive(true)
	end
	menu:UpdateButton(currentRole)
end

local function GetPlayerDetail(detail)
	local playerDetail = Inspect.Unit.Detail(player)
	if playerDetail then
		return playerDetail[detail]
	else
		return nil
	end
end

local function InFivePersonGroup(exact)
	local group = 0
	for i = 1, 20 do
		if Inspect.Unit.Lookup(string.format("group%02d", i)) then
			group = group + 1
			if (exact and i <=5 and group ~= i) or group > 5 then
				break
			end
		end
	end
	if exact then
		return group == 5
	else
		return group >= 1 and group <=5
	end
end

local function InDungeonGroup()
	local zone = GetPlayerDetail("zone")
	return zone and Inspect.Zone.Detail(zone).type == "instance" and InFivePersonGroup()
end

local checkRoleTypeTime = 0
local DelayedCheckRoleType

local function UpdateRoleType(type)
	if checkRoleTypeTime ~= 0 then
		Command.Event.Detach(Event.System.Update.Begin, DelayedCheckRoleType)
		checkRoleTypeTime = 0
	end
	type = type or GetPlayerDetail("role") or "none"
	if currentRole:GetType() ~= type and not InDungeonGroup() then
		currentRole:SetType(type)
		menu:UpdateButton(currentRole)
		UpdateMenu()
	end
end

DelayedCheckRoleType = function()
	if Inspect.Time.Real() > checkRoleTypeTime then
		if GetPlayerDetail("availability") == "full" then
			UpdateRoleType()
		else
			checkRoleTypeTime = Inspect.Time.Real() + 1
		end
	end
end

local function CheckRoleType(delay)
	if checkRoleTypeTime == 0 then
		checkRoleTypeTime = Inspect.Time.Real() + delay
		Command.Event.Attach(Event.System.Update.Begin, DelayedCheckRoleType, "Event.System.Update.Begin")
	else
		checkRoleTypeTime = math.max(checkRoleTypeTime, Inspect.Time.Real() + delay)
	end
end

function GetCurrentRole()
	return currentRole
end

local function SetCurrentRole(slot)
	if not roles[slot] then
		UpdateRoleList()
		UpdateMenu()
	end
	currentRole = roles[slot]
	menu:UpdateButton(currentRole)
	CheckRoleType(1)
end

local roleSlotChanged = 0
local soulWindowClosed = 0

Command.Event.Attach(Event.TEMPORARY.Role, function(_, slot)
	if player and currentRole ~= roles[slot] then
		roleSlotChanged = Inspect.Time.Real()
		SetCurrentRole(slot)
	end
end, "Event.TEMPORARY.Role")

UI.Native.Soul:EventAttach(Event.UI.Native.Loaded, function()
	if not UI.Native.Soul:GetLoaded() then
		soulWindowClosed = Inspect.Time.Real()
	end
end, "Event.UI.Native.Loaded")	

Command.Event.Attach(Event.Unit.Detail.Role, function(_, units)
	if units[player] ~= nil then
		local type = units[player]
		local time = Inspect.Time.Real()
		if type and ((time - roleSlotChanged > 1 and time - soulWindowClosed > 1) or InFivePersonGroup(true)) then
			CheckRoleType(2)
		else
			roleSlotChanged = 0
			soulWindowClosed = 0
			UpdateRoleType(type)
		end
	end
end, "Event.Unit.Detail.Role")

local function UnitAvailabilityFull(handle, units)
	for id, specifier in pairs(units) do
		if specifier == "player" then
			Command.Event.Detach(Event.Unit.Availability.Full, UnitAvailabilityFull)
			player = id
			SetCurrentRole(Inspect.TEMPORARY.Role())
			if addon:IsOutdated() then
				print(string.format(L.addon.newerVersion, addon.name))
			end
			print(string.format(L.addon.welcome, addon.name, addon.toc.Version))
			print(string.format(L.addon.instructions, "/roles"))
		end
	end
end
Command.Event.Attach(Event.Unit.Availability.Full, UnitAvailabilityFull, "Event.Unit.Availability.Full")

local function AddonSavedVariablesLoadEnd(_, identifier)
	if identifier == addon.identifier then
		Command.Event.Detach(Event.Addon.SavedVariables.Load.End, AddonSavedVariablesLoadEnd)
		local account = addon:GetSavedVariables("account")
		local character = addon:GetSavedVariables("character")

		-- Convert saved variables from pre-1.5 versions
		if not account.config and _G._config then
			account.config = _G._config
			_G._config = nil
			if account.config.height then
				account.config.spacing = account.config.height
				account.config.height = nil
			end
		end
		if not character.roles and _G._roles then
			character.roles = _G._roles
			_G._roles = nil
		end
		
		account.config = account.config or {}
		character.roles = character.roles or {}
		character.equipment = character.equipment or {}
		character.wardrobe = character.wardrobe or {}
		saved.config = account.config
		saved.roles = character.roles
		saved.equipment = character.equipment
		saved.wardrobe = character.wardrobe

		local configDefault =
		{
			sort = "Slot",
			lock = false,
			width = constants.defaultWidth,
			spacing = constants.defaultSpacing,
			extend = "Down",
			fade = constants.defaultFade,
			location = {},
			background = true,
			notification = true
		}
		Utility.SetDefaultValues(saved.config, configDefault)
		
		if not saved.config.lock then
			menu:EnableDrag(true, "Right")
		end
		
		saved.config.width = math.min(math.max(saved.config.width, constants.minWidth), constants.maxWidth)
		menu:SetWidth(saved.config.width)
		saved.config.spacing = math.min(math.max(saved.config.spacing, constants.minSpacing), constants.maxSpacing)
		menu:GetList():SetSpacing(saved.config.spacing)
		
		menu:SetFade(saved.config.fade)
		
		menu:SaveLocation(saved.config.location)
		if saved.config.location.x and saved.config.location.y then
			menu:SetLocation(saved.config.location.x, saved.config.location.y)
		else
			menu:Center()
		end
		
		menu:ShowBackground(saved.config.background)
		
		menu.OnMiddleClick = ShowConfig
		menu:EnableClick(true, "Middle")
	end
end
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, AddonSavedVariablesLoadEnd, "Event.Addon.SavedVariables.Load.End")

function SlashCommand(_, option)
	if option == "" then
		ShowConfig()
	elseif option == "toggle" then
		if Inspect.System.Secure() then
			print(L.command.errorCombat)
		elseif config:IsOpen() then
			print(L.command.errorConfig)
		else
			menu:Toggle()
		end
	elseif option == "lock" then
		menu:EnableDrag(saved.config.lock, "Right")
		saved.config.lock = not saved.config.lock
		UpdateLockButton()
		if saved.config.lock then
			print(L.command.statusLocked)
		else
			print(L.command.statusUnlocked)
		end
	elseif option == "reset" then
		menu:Center()
		print(L.command.statusReset)
	else
		print(L.command.options)
		print(string.format("/roles (%s)", L.command.optionConfig))
		print(string.format("/roles toggle (%s)", L.command.optionToggle))
		print(string.format("/roles lock (%s)", L.command.optionLock))
		print(string.format("/roles reset (%s)", L.command.optionReset))
	end
end
Command.Event.Attach(Command.Slash.Register("roles"), SlashCommand, "RolesSlashCommand")

function GetSetName(type, set)
	if saved[type][set] then
		return saved[type][set].name
	else
		return nil
	end
end

function SetSetName(type, set, name)
	if name then
		saved[type][set] = saved[type][set] or {}
		saved[type][set].name = name
	else
		saved[type][set] = nil
	end
end

local kAlert
function KaruulAlert()
	if kAlert == nil then
		if Inspect.Addon.List().kAlert then
			kAlert = true
		else
			kAlert = false
		end
	end
	return kAlert
end
